Object subclass: #SoapBroker
	instanceVariableNames: 'messageBindings exportSet soapMappingContainer responses '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapBroker methodsFor: 'accessing'!
envelope

	^envelope! !

!SoapBroker methodsFor: 'accessing'!
envelope: anObject

	envelope := anObject! !

!SoapBroker methodsFor: 'accessing'!
exportSet

	^exportSet! !

!SoapBroker methodsFor: 'accessing'!
exportSet: anObject

	exportSet := anObject! !

!SoapBroker methodsFor: 'accessing'!
messageBindings

	^messageBindings! !

!SoapBroker methodsFor: 'accessing'!
messageBindings: anObject

	messageBindings := anObject! !

!SoapBroker methodsFor: 'accessing'!
soapBindings

	^soapBindings! !

!SoapBroker methodsFor: 'accessing'!
soapBindings: anObject

	soapBindings := anObject! !

!SoapBroker methodsFor: 'accessing'!
soapDescriptions

	^soapDescriptions! !

!SoapBroker methodsFor: 'accessing'!
soapDescriptions: anObject

	soapDescriptions := anObject! !

!SoapBroker methodsFor: 'accessing'!
soapMappingContainer

	^soapMappingContainer! !

!SoapBroker methodsFor: 'accessing'!
soapMappingContainer: anObject

	soapMappingContainer := anObject! !


!SoapBroker methodsFor: 'initialize-release'!
initialize
	self initializeMessageBindings.
	self initializeExportedSet.
	self initializeMappingContainer! !

!SoapBroker methodsFor: 'initialize-release'!
initializeExportedSet
	self exportSet: Dictionary new! !

!SoapBroker methodsFor: 'initialize-release'!
initializeMappingContainer
	"Read in interfaceSpec for SoapMappingContainer and SoapDataType "
	| soapTypes xmlDoc |
	xmlDoc := self parser parse: self soapTypesXml.
	soapTypes := self soapTypesFrom: xmlDoc.
	self soapMappingContainer: (SoapMappingContainer new items: soapTypes)! !

!SoapBroker methodsFor: 'initialize-release'!
initializeMessageBindings
	self messageBindings: Dictionary new! !


!SoapBroker methodsFor: 'dispatching'!
dataTypeNamed: aName 
	^ self soapMappingContainer dataTypeNamed: aName ifAbsent: [(SoapError client detail: 'no such data type - ', aName) signal]! !

!SoapBroker methodsFor: 'dispatching'!
messageBindingNamed: aName 
	^ self messageBindings at: aName ifAbsent: [(SoapError client detail: 'no message binding - ', aName) signal]! !

!SoapBroker methodsFor: 'dispatching'!
objectExportedAs: key 
	^ [exportSet at: key ifAbsent: [(SoapError client detail: 'no such object - ', key ) signal]] valueUninterruptably! !


!SoapBroker methodsFor: 'public'!
addBinding: aSoapMessageBinding 
	"@aSoapDescription is either a SoapMethodDescription or a 	SoapResponseDescription "
	^ self messageBindings at: aSoapMessageBinding name put: aSoapMessageBinding! !

!SoapBroker methodsFor: 'public'!
addDescription: aSoapDescription 
	"@aSoapDescription is either a SoapMethodDescription or a 	SoapResponseDescription "
	^ self soapDescriptions add: aSoapDescription! !

!SoapBroker methodsFor: 'public'!
allActorsUsing: aSoapEnvelope 
	| anOrderedCollection |
	anOrderedCollection := OrderedCollection with: self defaultActor.
	anOrderedCollection addAll: aSoapEnvelope actors.
	^ anOrderedCollection! !

!SoapBroker methodsFor: 'public'!
defaultActor

	^SoapActor new! !

!SoapBroker methodsFor: 'public'!
defaultActorWith: aCollection

	^aCollection addFirst: self defaultActor; yourself! !

!SoapBroker methodsFor: 'public'!
export: anObject as: aKey 
	self exportSet at: aKey put: anObject! !

!SoapBroker methodsFor: 'public'!
extractEnvelopesFrom: aSoapEnvelope 
	"This method is to handle the situation where we have to forward part of the incoming envelop b/c 
	 
	we have actors in the soap header. we will return a collection that contains one or two SoapEnvelopes. 
	 
	if the collection contains one evelope it will be the original envelope.  
	if it contains two envelopes, then they will be two new envelopes, the first for our stuff, the second will be the stuff we forward."
	^ aSoapEnvelope containsActors
		ifTrue: [self halt]
		ifFalse: [OrderedCollection with: aSoapEnvelope]! !

!SoapBroker methodsFor: 'public'!
receive: aSoapEnvelope 
	| envelopes  envelope |
	"create a directed message from the @aSoapEnvelope and dispatch it 
	to the receiver. the receiver will be looked up in it's export set. "

	
	envelopes := self extractEnvelopesFrom: aSoapEnvelope.
	envelope := envelopes first. "current support only one"

	envelope broker: self.

	[envelope evaluate ] on: SoapError 
			do:[:ex | Transcript cr; show: '##SOAP ERROR## ', ex messageText.].

	^envelope! !

!SoapBroker methodsFor: 'public'!
receiveXml: xmlString
	
	| envelope str |

	envelope := SoapEnvelope fromXml: xmlString.
	str := WriteStream on: (String new: 1024).
	(self receive: envelope ) printXmlOn: str.
	^str contents.! !


!SoapBroker methodsFor: 'factory'!
parser
	"current not best but most standard Xml parser"
	^VWXMLXMLParser! !

!SoapBroker methodsFor: 'factory'!
soapTypesXml
	^self class soapTypesXml! !


!SoapBroker methodsFor: 'private'!
makeSetterMsgFrom: string
	^(string, ':') asSymbol! !

!SoapBroker methodsFor: 'private'!
soapTypeFrom: anXMLElement
	
	| soapType elements |
	elements := anXMLElement childNodes.
	soapType := SoapDataType new.
	elements do: [:each |
		| msg element |
		msg := self makeSetterMsgFrom: each tag type.
		element := each characterData.
		soapType perform: msg with: element].
	^soapType! !

!SoapBroker methodsFor: 'private'!
soapTypesFrom: xmlDoc
	"answer a collection of SoapDataType objects"

	| typeSection types | 
	typeSection := xmlDoc root.
	types := typeSection childNodes.
	^types collect: [:each |
		each tag type = 'SoapDataType'
			ifTrue: [self soapTypeFrom: each]].! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapBroker class
	instanceVariableNames: 'default '!

!SoapBroker class methodsFor: 'default values'!
default
	default isNil ifTrue: [default := self new].
	^ default! !

!SoapBroker class methodsFor: 'default values'!
soapTypesXml
	^ '<?xml version="1.0"?>
<SoapDataTypes>
<SoapDataType>
	<name>int</name>
	<smalltalkClass>Integer</smalltalkClass>
</SoapDataType>
<SoapDataType>
	<name>float</name>
	<smalltalkClass>Float</smalltalkClass>
</SoapDataType>
<SoapDataType>
	<name>string</name>
	<smalltalkClass>String</smalltalkClass>
</SoapDataType>

</SoapDataTypes>
'! !


!SoapBroker class methodsFor: 'class initialization'!
initialize
	"SoapBroker initialize"
	default := nil! !


!SoapBroker class methodsFor: 'tests'!
runBasicTest
	"SoapBroker runBasicTest"
	
	SoapBroker initialize.
	SoapBroker set4BasicTest.
	^SoapBroker default receiveXml: self simpleEnvelope.! !

!SoapBroker class methodsFor: 'tests'!
set4BasicTest
	| person soapBroker messageBinding |
	"SoapBroker set4BasicTest"
	person := Cs2Person new.
	person name: 'Adriaaaaan'; age: 25; height: 165; eyeColor: 'blue'.
	soapBroker := SoapBroker default export: person as: 'Person1234'.
	messageBinding := SoapMessageBinding new.
	messageBinding name: 'DoSomething'.
	messageBinding selector: #doSomething:.
	messageBinding addArgument: (SoapArgument
				name: 'symbol'
				dataType: (soapBroker dataTypeNamed: 'string')).	"this needs work"
	soapBroker addBinding: messageBinding.
	^soapBroker
	! !


!SoapBroker class methodsFor: 'fixtures'!
errorEnvelope
	
	^'

<SOAP-ENV:Envelope

	xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"

	SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">

	<SOAP-ENV:Body>

		<m:DoSomeOtherThing xmlns:m="Person1234">

			<symbol>DIS</symbol>

		</m:DoSomeOtherThing>

	</SOAP-ENV:Body>

</SOAP-ENV:Envelope>

'.
! !

!SoapBroker class methodsFor: 'fixtures'!
simpleEnvelope
	
	^'

<SOAP-ENV:Envelope

	xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"

	SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">

	<SOAP-ENV:Body>

		<m:DoSomething xmlns:m="Person1234">

			<symbol>DIS</symbol>

		</m:DoSomething>

	</SOAP-ENV:Body>

</SOAP-ENV:Envelope>

'.
! !


!SoapBroker class methodsFor: 'instance creation'!
new
	^ super new initialize! !


Object subclass: #SoapElement
	instanceVariableNames: 'dom broker '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapElement methodsFor: 'initialize-release'!
initialize
	"default do nothing"! !


!SoapElement methodsFor: 'printing'!
printXmlOn: stream
	^self dom printOn: stream! !

!SoapElement methodsFor: 'printing'!
printXmlString
	| stream |
	stream := WriteStream on: (String new: 1024).
	self dom printOn: stream.
	^stream contents.! !


!SoapElement methodsFor: 'actions'!
evaluate
	"default do nothing"! !


!SoapElement methodsFor: 'accessing'!
broker
	^broker! !

!SoapElement methodsFor: 'accessing'!
broker: aValue

	broker := aValue! !

!SoapElement methodsFor: 'accessing'!
dom
	dom isNil ifTrue: [dom := VWXMLElement tag: 'dummy'. "just for error proof"].
	^dom! !

!SoapElement methodsFor: 'accessing'!
dom: aDocumentObjectModel

	dom := aDocumentObjectModel! !


!SoapElement methodsFor: 'factory'!
parser
	
	^self class parser! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapElement class
	instanceVariableNames: ''!

!SoapElement class methodsFor: 'factory'!
parser
	"current not best but most standard Xml parser"
	^VWXMLXMLParser! !


!SoapElement class methodsFor: 'constants'!
tagName
	^'SOAP-ENV:', (self name copyReplaceAll: 'Soap' with: '')! !


!SoapElement class methodsFor: 'instance creation'!
fromXmlElement: anXmlElement
	^self new dom: anXmlElement! !

!SoapElement class methodsFor: 'instance creation'!
new
	^super new initialize! !


SoapElement subclass: #SoapActor
	instanceVariableNames: 'uri '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapActor methodsFor: 'accessing'!
uri

	^uri! !

!SoapActor methodsFor: 'accessing'!
uri: anObject

	uri := anObject! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapActor class
	instanceVariableNames: ''!

!SoapActor class methodsFor: 'instance creation'!
uri: aString

	^self new uri: aString! !


SoapElement subclass: #SoapArgument
	instanceVariableNames: 'dataType name value '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapArgument methodsFor: 'accessing'!
dataType

	^dataType! !

!SoapArgument methodsFor: 'accessing'!
dataType: anObject

	dataType := anObject! !

!SoapArgument methodsFor: 'accessing'!
name
	name isNil ifTrue:[ name := self dom tag asString].
	^name! !

!SoapArgument methodsFor: 'accessing'!
name: anObject

	name := anObject! !

!SoapArgument methodsFor: 'accessing'!
value
	value isNil ifTrue:[value := self dom characterData].
	^value! !

!SoapArgument methodsFor: 'accessing'!
value: anObject

	value := anObject! !


!SoapArgument methodsFor: 'actions'!
resolvedValue
	^ self dataType deserialize: self value! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapArgument class
	instanceVariableNames: ''!

!SoapArgument class methodsFor: 'instance creation'!
fromXmlElement: anXmlElement 
	^ (super fromXmlElement: anXmlElement) name: anXmlElement tag asString;
	 value: anXmlElement characterData! !

!SoapArgument class methodsFor: 'instance creation'!
name: aString dataType: aSoapDataType 
	^ self
		name: aString
		dataType: aSoapDataType
		value: nil! !

!SoapArgument class methodsFor: 'instance creation'!
name: aString dataType: aSoapDataType value: anObject 
	^ super new name: aString;
	 dataType: aSoapDataType;
	 value: anObject! !

!SoapArgument class methodsFor: 'instance creation'!
name: tagName value: aValue
	| dom |
	dom := (VWXMLElement tag: tagName elements: (Array with: (VWXMLText text: aValue))). 
	^self fromXmlElement: dom! !


SoapElement subclass: #SoapBody
	instanceVariableNames: 'messages fault '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapBody methodsFor: 'accessing'!
broker: aSoapBroker
	super broker: aSoapBroker.
	self messages do:[:each | each broker: self broker ].! !

!SoapBody methodsFor: 'accessing'!
fault
	fault isNil ifTrue: [ | elems |
	elems := (self dom selectNodes: [:each | each tag asString = SoapFault tagName]).
	fault := elems isEmpty ifTrue: [] ifFalse:[SoapFault fromXmlElement: elems first].].
	^ fault! !

!SoapBody methodsFor: 'accessing'!
fault: aValue
	fault := aValue! !

!SoapBody methodsFor: 'accessing'!
messages
	messages isNil ifTrue: [ | elems |
	elems := (self dom childNodes) select:[:each | each tag qualifier = 'm'].
	messages := elems collect:[:elem | SoapMessage fromXmlElement: elem].].
	^ messages! !

!SoapBody methodsFor: 'accessing'!
messages: aValue
	messages := aValue! !


!SoapBody methodsFor: 'testing'!
hasFault
	^self fault notNil! !


!SoapBody methodsFor: 'actions'!
becomeFaultBody: aSoapError
	 "quick hack for fault"
	| fa |
	fa := SoapFault fromSoapError: aSoapError.
	self messages: nil.
	self fault: fa.
	self dom removeAllNodes.
	self dom addNode: (fa dom).! !


SoapElement subclass: #SoapDataType
	instanceVariableNames: 'name smalltalkClass serializationBlock deserializationBlock '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapDataType methodsFor: 'platform specific'!
classFrom: string
	^Smalltalk at: string asSymbol 

! !


!SoapDataType methodsFor: 'initialize-release'!
initialize
	self serializationBlock: self defaultSerializationBlock;
	 deserializationBlock: self defaultDeserializationBlock! !


!SoapDataType methodsFor: 'accessing'!
deserializationBlock

	^deserializationBlock! !

!SoapDataType methodsFor: 'accessing'!
deserializationBlock: anObject

	deserializationBlock := anObject! !

!SoapDataType methodsFor: 'accessing'!
name

	^name! !

!SoapDataType methodsFor: 'accessing'!
name: aString 
	
	name := aString! !

!SoapDataType methodsFor: 'accessing'!
serializationBlock

	^serializationBlock! !

!SoapDataType methodsFor: 'accessing'!
serializationBlock: anObject

	serializationBlock := anObject! !

!SoapDataType methodsFor: 'accessing'!
smalltalkClass

	^smalltalkClass! !

!SoapDataType methodsFor: 'accessing'!
smalltalkClass: aString

	smalltalkClass := self classFrom: aString! !


!SoapDataType methodsFor: 'actions'!
defaultDeserializationBlock

	^[:aClass :aString | aClass fromString: aString ]! !

!SoapDataType methodsFor: 'actions'!
defaultSerializationBlock

	"should this return some kind of XML string "

	^[:value | value asString ]! !

!SoapDataType methodsFor: 'actions'!
deserialize: aString

	^self deserializationBlock value: self smalltalkClass value: aString! !


SoapElement subclass: #SoapEnvelope
	instanceVariableNames: 'header body '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapEnvelope methodsFor: 'accessing'!
body
	body isNil ifTrue: [ | elems |
	elems := (self dom selectNodes: [:each | each tag asString = SoapBody tagName]).
	body := elems isEmpty ifTrue: [] ifFalse:[SoapBody fromXmlElement: elems first]].
	^ body! !

!SoapEnvelope methodsFor: 'accessing'!
body: aSoapBody
	body := aSoapBody! !

!SoapEnvelope methodsFor: 'accessing'!
broker: aSoapBroker
	super broker: aSoapBroker.
	self header isNil ifFalse: [ self header broker: aSoapBroker].
	self body isNil ifFalse: [self body broker: aSoapBroker]! !

!SoapEnvelope methodsFor: 'accessing'!
header
	header isNil ifTrue: [ | elems |
	elems := (self dom selectNodes: [:each | each tag asString = SoapHeader tagName]).
	header := elems isEmpty ifTrue: [] ifFalse:[SoapHeader fromXmlElement: elems first]].
	^ header! !

!SoapEnvelope methodsFor: 'accessing'!
header: aSoapHeader
	header := aSoapHeader! !


!SoapEnvelope methodsFor: 'actions'!
actors

	| anOrderedCollection |
	anOrderedCollection := OrderedCollection new.
	^self header isNil
		ifTrue:[anOrderedCollection]
		ifFalse:[self extractActorsInto: anOrderedCollection]! !

!SoapEnvelope methodsFor: 'actions'!
evaluate
	[self body messages
		do: [:each | each evaluate]]
		on: SoapError
		do: [:ex | "quick hack for fault"
		self body becomeFaultBody: ex.
		ex pass]! !

!SoapEnvelope methodsFor: 'actions'!
extractActorsInto: aCollection

	" figure this out later use the header extract them"

	^aCollection! !

!SoapEnvelope methodsFor: 'actions'!
messages
	" self processHeaderInformation "
	^self body messages! !


!SoapEnvelope methodsFor: 'testing'!
containsActors
	^false! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapEnvelope class
	instanceVariableNames: ''!

!SoapEnvelope class methodsFor: 'instance creation'!
fromXml: xmlString
	^self fromXmlDocument: (self parser parse: xmlString).! !

!SoapEnvelope class methodsFor: 'instance creation'!
fromXmlDocument: anXmlDocument 
	"Create a new SoapEnvelope that wraps the passed XmlDocument 
	
	element or one of its children"
	"##cha"
	| element elements |
	(element := anXmlDocument root) tag asString = self tagName
		ifFalse: [(elements := element selectNodes: [:each | each tag asString = self tagName]) isEmpty ifFalse: [element := elements first]].
	^ (element isNil or: [element tag asString ~= self tagName])
		ifTrue: [self error: #'No SOAP Envelope']
		ifFalse: [self new dom: element]! !


Error subclass: #SoapError
	instanceVariableNames: 'faultCode '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapError methodsFor: 'accessing'!
detail
	^self messageText! !

!SoapError methodsFor: 'accessing'!
detail: aString
	self messageText: aString! !

!SoapError methodsFor: 'accessing'!
faultCode
	^faultCode! !

!SoapError methodsFor: 'accessing'!
faultCode: aValue
	faultCode := aValue! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapError class
	instanceVariableNames: ''!

!SoapError class methodsFor: 'defaults'!
client
	^self faultCode: #Client! !

!SoapError class methodsFor: 'defaults'!
mustUnderstand
	^self faultCode: #MustUnderstand! !

!SoapError class methodsFor: 'defaults'!
server
	^self faultCode: #Server! !

!SoapError class methodsFor: 'defaults'!
versionMismatch
	^self faultCode: #VersionMismatch! !


!SoapError class methodsFor: 'instance creation'!
faultCode: aFaultCodeSymbol
	^self new faultCode: aFaultCodeSymbol! !

!SoapError class methodsFor: 'instance creation'!
fromSoapFault: aSoapFault
	| inst |
	inst := self faultCode: aSoapFault faultcode.
	inst detail: aSoapFault detail.
	^inst ! !


SoapElement subclass: #SoapFault
	instanceVariableNames: 'faultcode faultstring faultactor detail '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapFault methodsFor: 'accessing'!
detail
	detail isNil
		ifTrue: [| det | 
			det := self dom childNodes
						detect: [:each | each tag type = 'detail']
						ifNone: [].
			det isNil
				ifFalse: [| msgElem | 
					msgElem := det childNodes first childNodes first.
					detail := msgElem characterData]].
	^ detail! !

!SoapFault methodsFor: 'accessing'!
detail: aValue

	detail := aValue! !

!SoapFault methodsFor: 'accessing'!
faultactor

	^faultactor! !

!SoapFault methodsFor: 'accessing'!
faultactor: aValue

	faultactor := aValue! !

!SoapFault methodsFor: 'accessing'!
faultcode
	faultcode isNil ifTrue: [ | elem |
		elem := self dom childNodes detect:[:each | each tag type = 'faultcode'] ifNone:[].
		faultcode := (elem characterData copyReplaceAll: 'SOAP-ENV:' with: '') asSymbol.
	].
	^faultcode! !

!SoapFault methodsFor: 'accessing'!
faultcode: aValue

	faultcode := aValue! !

!SoapFault methodsFor: 'accessing'!
faultstring
	faultstring isNil ifTrue: [ | elem |
		elem := self dom childNodes detect:[:each | each tag type = 'faultstring'] ifNone:[].
		faultstring := elem characterData.
	].
	^faultstring! !

!SoapFault methodsFor: 'accessing'!
faultstring: aValue

	faultstring := aValue! !


!SoapFault methodsFor: 'testing'!
isFault
	^true! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapFault class
	instanceVariableNames: ''!

!SoapFault class methodsFor: 'instance creation'!
fromSoapError: aSoapError
	| xml parsed |
	xml := aSoapError detail isNil ifTrue: [  self errorXmlOf: aSoapError faultCode ] 
								ifFalse:[ self errorXmlOf: aSoapError faultCode detail: aSoapError detail ].
	parsed := (self parser parse: xml) root.
	
	"VWXMLParser is too formal. So remove unneeded attributes"
	parsed attributes: nil.
	
	^self fromXmlElement: parsed! !


!SoapFault class methodsFor: 'utility'!
errorXmlOf: aSymbol
	^ '<', self tagName , ' xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
           <faultcode>SOAP-ENV:', aSymbol, '</faultcode>
           <faultstring>', aSymbol, ' Error</faultstring>' , '</', self tagName, '>'! !

!SoapFault class methodsFor: 'utility'!
errorXmlOf: aSymbol detail: aDetailString 
	^ '<', self tagName , ' xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
           <faultcode>SOAP-ENV:', aSymbol, '</faultcode>
           <faultstring>', aSymbol, ' Error</faultstring>
           	<detail>
                   <e:details xmlns:e="http://www.mars.dti.ne.jp/~umejava/smalltalk/soapOpera/">
                     <message>', aDetailString, '</message>
                   </e:details>
           </detail>
		' , '</', self tagName, '>'! !


SoapElement subclass: #SoapHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!


Object subclass: #SoapMappingContainer
	instanceVariableNames: 'items '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapMappingContainer methodsFor: 'accessing'!
items

	^items! !

!SoapMappingContainer methodsFor: 'accessing'!
items: anOrderedCollection

	items := anOrderedCollection! !


!SoapMappingContainer methodsFor: 'actions'!
dataTypeNamed: aName

	^self dataTypeNamed: aName ifAbsent: []! !

!SoapMappingContainer methodsFor: 'actions'!
dataTypeNamed: aName ifAbsent: aBlock

	^self items detect: [:each | each name = aName ] ifNone: [aBlock value]! !


SoapElement subclass: #SoapMessage
	instanceVariableNames: 'receiver arguments name result '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapMessage methodsFor: 'testing'!
isFault
	^false! !


!SoapMessage methodsFor: 'accessing'!
arguments
	arguments isNil ifTrue:[
		arguments :=  self dom childNodes collect: [:each | SoapArgument fromXmlElement: each ] 
	].
	^arguments! !

!SoapMessage methodsFor: 'accessing'!
arguments: aCollectionOfSoapArgument 
	self dom removeAllNodes.
	aCollectionOfSoapArgument
		do: [:each | self dom addNode: each dom].
	arguments := aCollectionOfSoapArgument! !

!SoapMessage methodsFor: 'accessing'!
name
	name isNil ifTrue: [
	name := (self dom tag type). "##must remove m:"].
	^name! !

!SoapMessage methodsFor: 'accessing'!
name: aString
	| tg |
	tg := self dom tag.
	tg
		qualifier: tg qualifier
		ns: tg namespace
		type: aString.
	name := aString! !

!SoapMessage methodsFor: 'accessing'!
receiver
	| mAttrib |
	mAttrib := VWXMLNodeTag new qualifier: 'xmlns' ns: '<!!-- xml namespace -->' type: 'm'.
	receiver isNil ifTrue: [receiver := self dom valueOfAttribute: mAttrib ifAbsent: []].
	^ receiver! !

!SoapMessage methodsFor: 'accessing'!
receiver: aString
	| att |
	att := self dom
				attributeNamed: (VWXMLNodeTag new
						qualifier: 'xmlns'
						ns: '<!!-- xml namespace -->'
						type: 'm').
	att value: aString.
	receiver := aString! !

!SoapMessage methodsFor: 'accessing'!
result
	"##needs more work##"
	result isNil ifTrue:[
		result :=  self dom childNodes first characterData.
	].
	^result! !

!SoapMessage methodsFor: 'accessing'!
result: anObject

	result := anObject! !


!SoapMessage methodsFor: 'converting'!
asDirectedMessage
	| aDirectedMessage aMessageBinding |
	aMessageBinding := self broker messageBindingNamed: self name.
	
	aDirectedMessage := MessageSend
				receiver: (self broker objectExportedAs: self receiver)
				selector: aMessageBinding selector
				arguments: (self argumentsUsing: aMessageBinding) asArray.
	^ aDirectedMessage! !


!SoapMessage methodsFor: 'actions'!
evaluate
	"raise error if evaluatein fails"
	| resultDom |
	[self result: self asDirectedMessage value]
		on: Error
		do: [:ex | (ex isKindOf: SoapError)
				ifTrue: [ex pass]
				ifFalse: [| rec | 
					rec := ex class name == #MessageNotUnderstood
								ifTrue: [' > ' , ex receiver printString , ' > ']
								ifFalse: [' > '].
					ex
						resignalAs: (SoapError server detail: ex class name , rec , ex description)]].
	"succeed - make response"
	self name: self name , 'Response'.
	resultDom := VWXMLElement
				tag: self result class name asString
				elements: (Array
						with: (VWXMLText text: self result storeString)).
	self dom removeAllNodes.
	self dom addNode: resultDom! !


!SoapMessage methodsFor: 'private'!
argumentsUsing: aMessageBinding
	"take an argument that was passed over the wire and reconcile 
	it to the 'template' argument that was bound in a message binding"
	^ self arguments
		collect: 
			[:unboundArgument | 
			| boundArgument | 
			boundArgument := aMessageBinding arguments detect: [:each | each name = unboundArgument name]
						ifNone: [self halt].
			unboundArgument dataType: boundArgument dataType.
			unboundArgument resolvedValue]! !


Object subclass: #SoapMessageBinding
	instanceVariableNames: 'arguments selector name '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Core'!

!SoapMessageBinding methodsFor: 'accessing'!
arguments

	arguments isNil ifTrue: [arguments := OrderedCollection new].

	^arguments! !

!SoapMessageBinding methodsFor: 'accessing'!
arguments: aCollection

	arguments := aCollection! !

!SoapMessageBinding methodsFor: 'accessing'!
name

	^name! !

!SoapMessageBinding methodsFor: 'accessing'!
name: anObject

	name := anObject! !

!SoapMessageBinding methodsFor: 'accessing'!
selector

	^selector! !

!SoapMessageBinding methodsFor: 'accessing'!
selector: anObject

	selector := anObject! !


!SoapMessageBinding methodsFor: 'initialize-release'!
initialize

	self initializeArguments.! !

!SoapMessageBinding methodsFor: 'initialize-release'!
initializeArguments

	self arguments: OrderedCollection new! !


!SoapMessageBinding methodsFor: 'actions'!
addArgument: aSoapArgument 
	"the order of the arguments is important. we will use this order 	to build the directed message. 
	we will lookup the arguments by name in the incoming domelement and convert the value based on the soapdatatype"
	
	^ self arguments addLast: aSoapArgument! !

!SoapMessageBinding methodsFor: 'actions'!
asDirectedMessage

	" will return a SoapMessage "! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapMessageBinding class
	instanceVariableNames: ''!

!SoapMessageBinding class methodsFor: 'instance creation'!
new

	^super new initialize! !

SoapBroker initialize!