Object subclass: #SoapHttpClient
	instanceVariableNames: ''
	classVariableNames: 'CrLf '
	poolDictionaries: ''
	category: 'SOAP-Http'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapHttpClient class
	instanceVariableNames: ''!

!SoapHttpClient class methodsFor: 'utility'!
forkSendTo: url request: xmlString
	
	^[self sendTo: url request: xmlString] fork! !

!SoapHttpClient class methodsFor: 'utility'!
sendTo: url request: soapEnvelopeXmlString
	
	"dirty, copied from HttpSocket"
	| serverName serverAddr s header bare page list firstData aStream port specifiedServer frm length newUrl con |

	Socket initializeNetwork.

	"parse url"
	bare := (url asLowercase beginsWith: 'http://') 
		ifTrue: [url copyFrom: 8 to: url size]
		ifFalse: [url].
	serverName := bare copyUpTo: $/.
	specifiedServer := serverName.
	(serverName includes: $:) ifFalse: [ port := '80' ] ifTrue: [
		port := (serverName copyFrom: (serverName indexOf: $:) + 1 
				to: serverName size) asNumber.
		serverName := serverName copyUpTo: $:.
	].

	(frm := (bare indexOf: $/) ) = 0 ifTrue:[frm := 1].
	page := bare copyFrom: frm to: bare size.
	page size = 0 ifTrue: [page := '/'].

  	"make the request"	
	HTTPSocket retry: [serverAddr := NetNameResolver addressForName: serverName timeout: 20.
				serverAddr ~~ nil] 
		asking: 'Trouble resolving server name.  Keep trying?'
		ifGiveUp: [^ 'Could not resolve the server named: ', serverName].

	s := HTTPSocket new.
	s connectTo: serverAddr port: port.
	s waitForConnectionUntil: Socket standardDeadline.
	s sendCommand: 'POST ', page, ' HTTP/1.0', CrLf, 
		'ACCEPT: text/xml', CrLf,
		'User-Agent: ', Smalltalk version, CrLf,
		'Content-type: text/xml', CrLf,
		'Content-length: ', soapEnvelopeXmlString size printString, CrLf,
		'Host: ', specifiedServer, 
		'SOAPAction:', CrLf, CrLf, soapEnvelopeXmlString.

	"get the header of the reply"
	list := s getResponseUpTo: CrLf, CrLf ignoring: (String with: Character cr).	"list = header, CrLf, CrLf, beginningOfData"
	header := list at: 1.
	firstData := list at: 3.
	
	"Transcript cr; show: 'header: ', header, ' data: ', firstData."

	"dig out some headers"
	s header: header.
	length := s getHeader: 'content-length'.
	length ifNotNil: [ length := length asNumber ].
	s responseCode first = $3 ifTrue: [
		newUrl := s getHeader: 'location'.
		newUrl ifNotNil: [
			Transcript show: 'redirecting to: ', newUrl; cr.
			s destroy.
			^self sendTo: newUrl request: soapEnvelopeXmlString ] ].

	aStream := s getRestOfBuffer: firstData totalLength: length.
	con := aStream contents.
	s responseCode = '401' ifTrue: [^ header, con].
	s destroy.	"Always OK to destroy!!"
	^con
	
	 ! !


!SoapHttpClient class methodsFor: 'class initialization'!
initialize
	"SoapClient initialize"
	CrLf := String with: Character cr with: Character lf.! !


!SoapHttpClient class methodsFor: 'tests'!
test1
	"SoapHttpClient test1 "
	"(print it)"
	^SoapHttpClient sendTo: 'localhost:8823' request: SoapBroker simpleEnvelope! !

!SoapHttpClient class methodsFor: 'tests'!
test2
	"SoapHttpClient test2"
	"(print it)"
	^SoapHttpClient sendTo: 'localhost:8823' request: SoapBroker errorEnvelope! !

!SoapHttpClient class methodsFor: 'tests'!
testInit
	"SoapHttpClient testInit "
	SoapBroker initialize.
	SoapBroker set4BasicTest.! !


Object subclass: #SoapHttpModule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Http'!

!SoapHttpModule methodsFor: 'processing'!
process: request
	| rawXml ret |
	
	(request contentType = MIMEDocument contentTypeXml)
	ifFalse:[^nil].
	
	rawXml := request stream next: request contentLength. "body"

	self class debugMode ifTrue: [Transcript show: 'reqBody: ', rawXml printString. "##for debug##"].
	
	ret := SoapBroker default receiveXml: rawXml.
	
	^HttpResponse fromMIMEDocument: 
		(MIMEDocument contentType: 'text/xml' content: ret)
	
	
	
	! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapHttpModule class
	instanceVariableNames: 'debugMode '!

!SoapHttpModule class methodsFor: 'instance creation'!
soapHttpServer
	| instance |
	instance := self newModuleNamed: 'SOAP'.
	^instance! !


!SoapHttpModule class methodsFor: 'class initialization'!
initialize
	"SoapHttpModule initialize"
	debugMode := false! !


!SoapHttpModule class methodsFor: 'accessing'!
debugMode
	debugMode isNil ifTrue:[debugMode := false].
	^debugMode! !

!SoapHttpModule class methodsFor: 'accessing'!
debugMode: aBoolean
	debugMode := aBoolean! !


ComancheNetService subclass: #SoapHttpService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Http'!

!SoapHttpService methodsFor: 'accessing'!
module
	module isNil ifTrue: [ module := self defaultModule].
	^module! !


!SoapHttpService methodsFor: 'defaults'!
defaultModule
	^SoapHttpModule new! !
 
Object subclass: #SoapHttpSetup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Http'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapHttpSetup class
	instanceVariableNames: 'service '!

!SoapHttpSetup class methodsFor: 'class initialization'!
initialize
	"SoapHttpSetup initialize"
	Smalltalk addToStartUpList: self! !

!SoapHttpSetup class methodsFor: 'class initialization'!
release
	service := nil! !


!SoapHttpSetup class methodsFor: 'utility'!
launch
	"For GUI setup"
	self error: '0.2b temporarily does not support GUI setup, sorry.'! !

!SoapHttpSetup class methodsFor: 'utility'!
start
	"SoapHttpSetup start"
	^self service start! !

!SoapHttpSetup class methodsFor: 'utility'!
stop
	"SoapHttpSetup stop"
	self service unregister.
	self release! !


!SoapHttpSetup class methodsFor: 'accessing'!
service
	service isNil ifTrue: [ service := self defaultService ].
	^service! !

!SoapHttpSetup class methodsFor: 'accessing'!
service: aSoapHttpService
	
	service := aSoapHttpService! !


!SoapHttpSetup class methodsFor: 'defaults'!
defaultService
	^SoapHttpService named: 'SOAP' onPort: OperaSetting default defaultPort.! !


!SoapHttpSetup class methodsFor: 'start up/shut down'!
startUp
	self stop.
	self start! !

SoapHttpClient initialize!
SoapHttpModule initialize!
SoapHttpSetup initialize!