Object subclass: #NamedObjectHome
	instanceVariableNames: 'objectDict tempObjectCounter '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!NamedObjectHome methodsFor: 'actions'!
at: aSymbol
	
	^ objectDict at: aSymbol ifAbsent:[(OperaError noImplement: aSymbol) signal]! !

!NamedObjectHome methodsFor: 'actions'!
at: aSymbol ifAbsent: aBlock
	^ objectDict at: aSymbol ifAbsent: aBlock! !

!NamedObjectHome methodsFor: 'actions'!
at: aSymbol ifAbsentPut: aBlock
	self checkCleanTempObjects.
	^ objectDict at: aSymbol ifAbsentPut: aBlock! !

!NamedObjectHome methodsFor: 'actions'!
at: aSymbol put: anObject 
	self checkCleanTempObjects.
	^ objectDict at: aSymbol put: anObject! !

!NamedObjectHome methodsFor: 'actions'!
removeKey: aSymbol
	^ objectDict removeKey: aSymbol ! !

!NamedObjectHome methodsFor: 'actions'!
removeKey: aSymbol ifAbsent: aBlock
	^ objectDict removeKey: aSymbol ifAbsent: aBlock! !


!NamedObjectHome methodsFor: 'initialize-release'!
initialize
	objectDict := IdentityDictionary new.
	tempObjectCounter := 0.! !


!NamedObjectHome methodsFor: 'printing'!
printOn: aStream
	aStream nextPutAll:'objectHome:'.
	aStream cr. 
	objectDict printOn: aStream! !


!NamedObjectHome methodsFor: 'factory'!
defaultDictionary
	^IdentityDictionary new! !


!NamedObjectHome methodsFor: 'private'!
checkCleanTempObjects
	tempObjectCounter := tempObjectCounter + 1.
	tempObjectCounter > 9999
		ifTrue: 
			[[objectDict keysAndValuesRemove: [:key :value | key beginsWith: #temp:]] valueUninterruptably.
			tempObjectCounter := 0]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NamedObjectHome class
	instanceVariableNames: ''!

!NamedObjectHome class methodsFor: 'instance creation'!
new
	^super new initialize! !


Error subclass: #OperaError
	instanceVariableNames: 'originalError minorCode description '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!OperaError methodsFor: 'accessing'!
description
	description isNil ifTrue: [description := super description].
	^description! !

!OperaError methodsFor: 'accessing'!
description: aValue

	description := aValue! !

!OperaError methodsFor: 'accessing'!
minorCode

	^minorCode! !

!OperaError methodsFor: 'accessing'!
minorCode: aValue

	minorCode := aValue! !

!OperaError methodsFor: 'accessing'!
originalError

	^originalError! !

!OperaError methodsFor: 'accessing'!
originalError: aValue

	originalError := aValue! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OperaError class
	instanceVariableNames: ''!

!OperaError class methodsFor: 'defaults'!
badParameter
	^self new messageText: #'BAD_PARAM'! !

!OperaError class methodsFor: 'defaults'!
noImplement
	^self new messageText: #'NO_IMPLEMENT'! !

!OperaError class methodsFor: 'defaults'!
noImplement: anObject
	| noImpl |
	noImpl := self noImplement.
	noImpl messageText: (noImpl messageText, ' ', anObject printString).
	^noImpl! !

!OperaError class methodsFor: 'instance creation'!
fromSoapError: aSoapError
	| inst |
	"##needs more work##"
	inst := self new.
	inst originalError: aSoapError.
	inst description: (aSoapError description copyReplaceAll: 'SoapError:' with: 'OperaError:').
	inst messageText: aSoapError messageText.
	inst minorCode: #Unknown.
	^inst! !


Object subclass: #OperaORB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OperaORB class
	instanceVariableNames: ''!

!OperaORB class methodsFor: 'accessing'!
rootOOA
	^OperaObjectAdapter root! !


!OperaORB class methodsFor: 'utility'!
debugMode
	^OperaSetting default debugMode! !

!OperaORB class methodsFor: 'utility'!
debugMode: aSymbol 
	| mode |
	"#http, #object, #off are supported"
	OperaSetting default debugMode: aSymbol.
	mode := OperaSetting default debugMode.
	mode == #http ifTrue: [SoapHttpModule debugMode: true].
	(mode == #off or: [mode == #object]) ifTrue: [SoapHttpModule debugMode: false]. ! !

!OperaORB class methodsFor: 'utility'!
launch
	"##need more work##"
	SoapHttpSetup launch.! !


!OperaORB class methodsFor: 'testing'!
isRunning
	^SoapHttpSetup service isRunning! !


!OperaORB class methodsFor: 'actions'!
init
	OperaSetting default initialize.
	! !

!OperaORB class methodsFor: 'actions'!
init: aPort
	OperaSetting default initialize.
	OperaSetting default defaultPort: aPort! !

!OperaORB class methodsFor: 'actions'!
run
	"OperaORB run"
	SoapHttpSetup stop.
	SoapHttpSetup start! !

!OperaORB class methodsFor: 'actions'!
shutdown
	"OperaORB shutdown"
	SoapHttpSetup stop! !


!OperaORB class methodsFor: 'backward compatibility'!
start
	"OperaORB start"
	"Backward compatibility"
	self run! !


Object subclass: #OperaObjectReference
	instanceVariableNames: 'name host port impl '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!OperaObjectReference methodsFor: 'accessing'!
host
	
	^host! !

!OperaObjectReference methodsFor: 'accessing'!
host: aValue

	host := aValue! !

!OperaObjectReference methodsFor: 'accessing'!
impl
	
	^impl! !

!OperaObjectReference methodsFor: 'accessing'!
impl: aValue

	impl := aValue! !

!OperaObjectReference methodsFor: 'accessing'!
name

	^name! !

!OperaObjectReference methodsFor: 'accessing'!
name: aValue

	name := aValue! !

!OperaObjectReference methodsFor: 'accessing'!
port
	
	^port! !

!OperaObjectReference methodsFor: 'accessing'!
port: aValue

	port := aValue! !


!OperaObjectReference methodsFor: 'initialize-release'!
initialize
	host := OperaSetting default defaultCallbackHost.
	port := OperaSetting default defaultPort.! !


!OperaObjectReference methodsFor: 'object from disk'!
readDataFrom: aDataStream size: varsOnDisk 
	| cntInstVars |
	cntInstVars := varsOnDisk.
	"OK if fewer than now"
	aDataStream beginReference: self.
	1 to: cntInstVars do: [:i | self instVarAt: i put: aDataStream next]! !

!OperaObjectReference methodsFor: 'object from disk'!
storeDataOn: aDataStream 
	| cntInstVars |
	cntInstVars := self class instSize - 1.
	"remove impl"
	aDataStream beginInstance: self class size: cntInstVars.
	1 to: cntInstVars do: [:i | aDataStream nextPut: (self instVarAt: i)]! !


!OperaObjectReference methodsFor: 'comparing'!
= anOperaObjectReference 
	self name = anOperaObjectReference name ifFalse: [^ false].
	self host = anOperaObjectReference host ifFalse: [^ false].
	self port = anOperaObjectReference port ifFalse: [^ false].
	^ true! !

!OperaObjectReference methodsFor: 'comparing'!
hash
	^ self name hash! !


!OperaObjectReference methodsFor: 'converting'!
asRemoteObject
	| remoteObjClass |
	remoteObjClass := OperaSetting default debugMode == #object ifTrue: [DebugRemoteObject] ifFalse:[RemoteObject].
	^remoteObjClass name: (self name) host: (self host) port: (self port) ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OperaObjectReference class
	instanceVariableNames: ''!

!OperaObjectReference class methodsFor: 'instance creation'!
new
	^super new initialize! !

!OperaObjectReference class methodsFor: 'instance creation'!
on: anObject
	^self new impl: anObject! !


!OperaObjectReference class methodsFor: 'comparing'!
hasInstance: anObject
	"performance conscious comparison"
	^anObject class name == self name! !


ReferenceStream subclass: #OperaObjectStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OperaObjectStream class
	instanceVariableNames: ''!

!OperaObjectStream class methodsFor: 'marshal/unmarshal'!
objectFromString: aString
	| byteStr oStr |
	byteStr := (Base64MimeConverter mimeDecodeToBytes: (ReadStream on: aString)).
	oStr := self on: byteStr.
	^oStr next! !

!OperaObjectStream class methodsFor: 'marshal/unmarshal'!
stringFromObject: anObject 
	| file con |
	file := RWBinaryOrTextStream
				on: (ByteArray new: 5000).
	file binary.
	(self on: file)
		nextPut: anObject.
	con := Base64MimeConverter mimeEncode: file reset.
	file close.
	^ con contents! !


Object subclass: #OperaOidGenerator
	instanceVariableNames: 'highValue lowValue highValueCounter '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!OperaOidGenerator methodsFor: 'initialize-release'!
initialize
	highValueCounter := 0.! !


!OperaOidGenerator methodsFor: 'accessing'!
highValue

	highValue isNil ifTrue:[ highValue := self privNextValue. ].

	^highValue! !

!OperaOidGenerator methodsFor: 'accessing'!
highValue: aValue

	highValue := aValue! !

!OperaOidGenerator methodsFor: 'accessing'!
highValueCounter
	^highValueCounter! !

!OperaOidGenerator methodsFor: 'accessing'!
highValueCounter: aValue

	highValueCounter := aValue! !

!OperaOidGenerator methodsFor: 'accessing'!
lowValue

	lowValue isNil ifTrue:[ lowValue := self privNextValue. ].

	^lowValue! !

!OperaOidGenerator methodsFor: 'accessing'!
lowValue: aValue

	lowValue := aValue! !


!OperaOidGenerator methodsFor: 'private'!
nextHighValue
	self highValueCounter: self highValueCounter + 1.
	self highValueCounter > 9999
		ifTrue: 
			[self highValueCounter: 0.
			self highValue: self privNextValue].
	^ self highValue.! !

!OperaOidGenerator methodsFor: 'private'!
nextLowValue
	self lowValue: self privNextValue.
	^ self lowValue.! !

!OperaOidGenerator methodsFor: 'private'!
privNextValue
	| sz str |
	sz := 6.
	str := String new: sz.
	1 to: sz do:[ :idx |
	str at: idx put: 15 atRandom asHexDigit]. 
	^str
	
	! !


!OperaOidGenerator methodsFor: 'actions'!
nextValue
	^(self nextHighValue , self nextLowValue)! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OperaOidGenerator class
	instanceVariableNames: ''!

!OperaOidGenerator class methodsFor: 'instance creation'!
new
	^super new initialize! !


Object subclass: #OperaSingleton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!OperaSingleton methodsFor: 'initialize-release'!
initialize
	"default do nothing"! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OperaSingleton class
	instanceVariableNames: 'default '!

!OperaSingleton class methodsFor: 'class initialization'!
initialize
	
	default := nil.
	! !

!OperaSingleton class methodsFor: 'class initialization'!
initializeAll
	"OperaSingleton initializeAll"
	
	OperaSingleton allSubclassesDo:[:each | each initialize]
	! !


!OperaSingleton class methodsFor: 'instance creation'!
default
	default isNil ifTrue:[default := self new].
	^default! !

!OperaSingleton class methodsFor: 'instance creation'!
new
	^super new initialize! !


OperaSingleton subclass: #OperaConnector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!OperaConnector methodsFor: 'private'!
processArguments: aCollectionOfArguments
	"##needs more work##"
	"currently obj ref argument is always registerd in root ooa"
	aCollectionOfArguments do:[:each |
		(OperaObjectReference hasInstance: each)
			ifTrue:[ OperaORB rootOOA registerObjectImplFor: each.
					"passing obj ref to server, so impl should be nil"
					each impl: nil.
			]
	].
	^aCollectionOfArguments! !

!OperaConnector methodsFor: 'private'!
processReturn: aReturnedXmlString 
	| env msg obj |
	env := SoapEnvelope fromXml: aReturnedXmlString.
	env body hasFault
		ifTrue: [self signalErrorFrom: env body fault].
	msg := env body messages first. "currently only one msg support"

	obj := OperaObjectStream objectFromString: msg result. 
	obj class name == #OperaObjectReference ifTrue:[ obj := obj asRemoteObject]. "sort of obj ref support"
	^obj! !

!OperaConnector methodsFor: 'private'!
signalErrorFrom: aSoapFault
	 | ex |
	ex := (OperaError fromSoapError: (SoapError fromSoapFault: aSoapFault)).
	ex signal! !

!OperaConnector methodsFor: 'private'!
wrapSoapEnvelope: aRemoteMessageSend 
	| env msg |
	env := SoapEnvelope fromXml: self class templateEnvelopeXml.
	msg := env body messages first.
	"##need more work##"
	"We need XML schema marshal/unmarshaller for true SOAP interoperability"
	msg name: OperaObjectAdapter dispatchSelectorName.
	msg receiver: OperaObjectAdapter serverURIName.
	msg
		arguments: (Array
				with: (SoapArgument name: aRemoteMessageSend class name asString value: aRemoteMessageSend storeString)).
	^ env! !


!OperaConnector methodsFor: 'sending'!
send: aRemoteMessageSend to: host port: port
	| url env ret |
	
	self processArguments: aRemoteMessageSend arguments.
	
	url := host, ':', port asString.
	env := self wrapSoapEnvelope: aRemoteMessageSend.
	ret := SoapHttpClient sendTo: url request: env printXmlString.

	"Transcript show: ret printString." "##for debug##"
	^self processReturn: ret.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OperaConnector class
	instanceVariableNames: ''!

!OperaConnector class methodsFor: 'constants'!
templateEnvelopeXml
	^'
<SOAP-ENV:Envelope
	xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
	SOAP-ENV:encodingStyle="http://www.mars.dti.ne.jp/~umejava/soapOpera/squeakEncoding/">
	<SOAP-ENV:Body>
		<m:selector xmlns:m="receiver">
			<symbol>value</symbol>
		</m:selector>
	</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
'.! !


OperaSingleton subclass: #OperaObjectAdapter
	instanceVariableNames: 'namedObjectHome oidGenerator '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!OperaObjectAdapter methodsFor: 'tiny naming service'!
at: aSymbol
	^ self namedObjectHome at: aSymbol! !

!OperaObjectAdapter methodsFor: 'tiny naming service'!
at: aSymbol ifAbsent: aBlock
	^ self namedObjectHome at: aSymbol ifAbsent: aBlock! !

!OperaObjectAdapter methodsFor: 'tiny naming service'!
at: aSymbol ifAbsentPut: aBlock
	^ self namedObjectHome at: aSymbol ifAbsentPut: aBlock! !

!OperaObjectAdapter methodsFor: 'tiny naming service'!
at: aSymbol put: anObject 
	^ self namedObjectHome at: aSymbol put: anObject! !

!OperaObjectAdapter methodsFor: 'tiny naming service'!
removeKey: aSymbol
	^ self namedObjectHome removeKey: aSymbol ! !

!OperaObjectAdapter methodsFor: 'tiny naming service'!
removeKey: aSymbol ifAbsent: aBlock
	^ self namedObjectHome removeKey: aSymbol ifAbsent: aBlock! !


!OperaObjectAdapter methodsFor: 'accessing'!
namedObjectHome
	namedObjectHome isNil ifTrue:[namedObjectHome := NamedObjectHome new].
	^namedObjectHome! !

!OperaObjectAdapter methodsFor: 'accessing'!
namedObjectHome: aNamedObjectHome
	namedObjectHome := aNamedObjectHome
	! !

!OperaObjectAdapter methodsFor: 'accessing'!
oidGenerator
	oidGenerator isNil ifTrue:[oidGenerator := OperaOidGenerator new].
	^oidGenerator! !

!OperaObjectAdapter methodsFor: 'accessing'!
oidGenerator: aNamedObjectHome
	oidGenerator := aNamedObjectHome
	! !


!OperaObjectAdapter methodsFor: 'private'!
activateObjectReference: anObjectReference named: aRegisteredObjectName 
	| found |
	"assign object name"
	"implicitly create unshared obj ref name and register"
	found := self at: aRegisteredObjectName ifAbsent: [].
	found isNil
		ifTrue: 
			[|  impl nm | 
			impl := anObjectReference impl.
			nm := self nextTempObjectNameFor: impl.
			self at: nm put: impl. 
			anObjectReference name: nm]
		ifFalse: [anObjectReference name: aRegisteredObjectName]. 
	^ anObjectReference! !

!OperaObjectAdapter methodsFor: 'private'!
nextTempObjectName
	^('temp:', self oidGenerator nextValue) asSymbol! !

!OperaObjectAdapter methodsFor: 'private'!
nextTempObjectNameFor: anObject
	| wStr |
	"^('temp:', self oidGenerator nextValue,':', anObject name) asSymbol"
	wStr := WriteStream on: (String new: 32).
	wStr nextPutAll: 'temp:'.
	wStr nextPutAll: self oidGenerator nextValue.
	wStr nextPutAll: ':'.
	wStr nextPutAll: anObject class name.
	^wStr contents asSymbol! !


!OperaObjectAdapter methodsFor: 'initialize-release'!
initialize
	oidGenerator := nil.
	self initializeNamedObjectHome! !

!OperaObjectAdapter methodsFor: 'initialize-release'!
initializeNamedObjectHome
	
	self namedObjectHome initialize! !


!OperaObjectAdapter methodsFor: 'receiving'!
dispatch: aStringfiedRemoteMessageSend 
	| message ret |
	message := RemoteMessageSend fromString: aStringfiedRemoteMessageSend.
	message namedObjectHome: self namedObjectHome.
	message arguments
		withIndexDo: [:each :idx | (OperaObjectReference hasInstance: each)
				ifTrue: [message arguments at: idx put: each asRemoteObject]].

	"##need more work##"
	"need error handling for obj ref"
	ret := RemoteReturnValue value: message value.

	"##restrict implicit self return-by-value##"
	ret value == message receiver ifTrue: [^ ret value: (self activateObjectReference: (OperaObjectReference on: ret value)
				named: message receiverName)].

	"##investigate return-by-value or return-by-ref##"
	(OperaObjectReference hasInstance: ret value)
		ifTrue: [^ ret value: (self activateObjectReference: ret value named: ret value name)].

	^ ret! !


!OperaObjectAdapter methodsFor: 'registering'!
registerObjectImplFor: anObjectReference
	| found |
	(OperaObjectReference hasInstance: anObjectReference) ifFalse:[OperaError badParameter; description:'parameter is not object refrence'; signal].
	
	"if found, object has been registered already, so do nothing"
	
	found := self at: anObjectReference name ifAbsent: [].
	found isNil
		ifTrue: 
			[| nm impl |
			impl :=  anObjectReference impl.
			nm := self nextTempObjectNameFor: impl.
			self at: nm put: impl.
			anObjectReference name: nm].

	^ anObjectReference! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OperaObjectAdapter class
	instanceVariableNames: ''!

!OperaObjectAdapter class methodsFor: 'class initialization'!
initialize
	"OperaObjectAdapter initialize"
	super initialize.
	self installRootOOA
	! !

!OperaObjectAdapter class methodsFor: 'class initialization'!
installRootOOA
	| acceptor soapBroker messageBinding |
	acceptor := self root.
	soapBroker := SoapBroker default export: acceptor as: (self serverURIName).
	messageBinding := SoapMessageBinding new.
	messageBinding name: (self dispatchSelectorName).
	messageBinding selector: #dispatch:.
	messageBinding addArgument: (SoapArgument
				name: 'RemoteMessageSend'
				dataType: (soapBroker dataTypeNamed: 'string')).
	soapBroker addBinding: messageBinding.
	! !


!OperaObjectAdapter class methodsFor: 'constants'!
dispatchSelectorName
	^'Dispatch'! !

!OperaObjectAdapter class methodsFor: 'constants'!
serverName
	^'RootOOA'! !

!OperaObjectAdapter class methodsFor: 'constants'!
serverURIName
	^'urn:SoapOpera:', self serverName! !


!OperaObjectAdapter class methodsFor: 'instance creation'!
root
	"OperaObjectAdapter root"
	"Returns RootOOA"
	^super default! !


OperaSingleton subclass: #OperaSetting
	instanceVariableNames: 'defaultHost defaultPort defaultCallbackHost debugMode '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!OperaSetting methodsFor: 'accessing'!
defaultCallbackHost
	defaultCallbackHost isNil ifTrue:[defaultCallbackHost := NetNameResolver localAddressString].
	^defaultCallbackHost! !

!OperaSetting methodsFor: 'accessing'!
defaultCallbackHost: aValue

	defaultCallbackHost := aValue! !
	
!OperaSetting methodsFor: 'accessing'!
debugMode
	debugMode isNil ifTrue:[debugMode := #off].
	^debugMode! !

!OperaSetting methodsFor: 'accessing'!
debugMode: aSymbol
	"#http, #object, #off are supported"
	debugMode := aSymbol! !

!OperaSetting methodsFor: 'accessing'!
defaultHost
	defaultHost isNil ifTrue:[defaultHost := 'localhost'].
	^defaultHost! !

!OperaSetting methodsFor: 'accessing'!
defaultHost: aValue

	defaultHost := aValue! !

!OperaSetting methodsFor: 'accessing'!
defaultPort
	defaultPort isNil ifTrue:[defaultPort := 8823].
	^defaultPort! !

!OperaSetting methodsFor: 'accessing'!
defaultPort: aValue

	defaultPort := aValue! !


MessageSend subclass: #RemoteMessageSend
	instanceVariableNames: 'receiverName namedObjectHome value '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!RemoteMessageSend methodsFor: 'accessing'!
namedObjectHome
	namedObjectHome isNil ifTrue:[namedObjectHome := NamedObjectHome new].
	^namedObjectHome! !

!RemoteMessageSend methodsFor: 'accessing'!
namedObjectHome: aNamedObjectHome
	namedObjectHome := aNamedObjectHome! !

!RemoteMessageSend methodsFor: 'accessing'!
receiverName
	^receiverName! !

!RemoteMessageSend methodsFor: 'accessing'!
receiverName: anObjectNameSymbol
	receiverName := anObjectNameSymbol! !


!RemoteMessageSend methodsFor: 'storing'!
storeString
	^OperaObjectStream stringFromObject: self! !


!RemoteMessageSend methodsFor: 'private'!
receiverOf: aObjectName
	^self namedObjectHome at: aObjectName! !


!RemoteMessageSend methodsFor: 'actions'!
receiver
	^(self receiverOf: self receiverName)! !


!RemoteMessageSend methodsFor: 'evaluating'!
value
	"evaluation is occurred only once"
	value isNil ifTrue: [value := arguments isNil
					ifTrue: [self receiver perform: selector]
					ifFalse: [self receiver perform: selector withArguments: arguments]].
	^ value! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RemoteMessageSend class
	instanceVariableNames: ''!

!RemoteMessageSend class methodsFor: 'instance creation'!
fromString: aString
	 ^OperaObjectStream objectFromString: aString.! !

!RemoteMessageSend class methodsFor: 'instance creation'!
receiverName: anObject selector: aSymbol
	^ self receiverName: anObject selector: aSymbol arguments: #()! !

!RemoteMessageSend class methodsFor: 'instance creation'!
receiverName: anObject selector: aSymbol argument: aParameter
	^ self receiverName: anObject selector: aSymbol arguments: (Array with: aParameter)! !

!RemoteMessageSend class methodsFor: 'instance creation'!
receiverName: anObject selector: aSymbol arguments: anArray
	^ self new
		receiverName: anObject;
		selector: aSymbol;
		arguments: anArray! !


!RemoteMessageSend class methodsFor: 'constants'!
tagName
	^self name! !


ProtoObject subclass: #RemoteObject
	instanceVariableNames: 'name host port '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!RemoteObject methodsFor: 'accessing'!
oprHost
	host isNil ifTrue:[host := OperaSetting default defaultHost].
	^host! !

!RemoteObject methodsFor: 'accessing'!
oprHost: aValue

	host := aValue! !

!RemoteObject methodsFor: 'accessing'!
oprName
	name isNil ifTrue:[name := 'server'].
	^name! !

!RemoteObject methodsFor: 'accessing'!
oprName: aValue

	name := aValue! !

!RemoteObject methodsFor: 'accessing'!
oprPort
	port isNil ifTrue:[port := OperaSetting default defaultPort].
	^port	! !

!RemoteObject methodsFor: 'accessing'!
oprPort: aValue

	port := aValue! !


!RemoteObject methodsFor: 'special'!
= anOperaObjectReference 
	self oprName = anOperaObjectReference oprName ifFalse: [^ false].
	self oprHost = anOperaObjectReference oprHost ifFalse: [^ false].
	self oprPort = anOperaObjectReference oprPort ifFalse: [^ false].
	^ true! !

!RemoteObject methodsFor: 'special'!
halt
	"useful for debug"

	Halt signal! !

!RemoteObject methodsFor: 'special'!
hash
	^ self oprPrintString hash! !

!RemoteObject methodsFor: 'special'!
inspect
	"useful for debug"
	| dic |
	dic := IdentityDictionary new.
	dic at: #ClassName put: #RemoteObject.
	dic at: #name put: self oprName.
	dic at: #host put: self oprHost.
	dic at: #port put: self oprPort.
	Inspector openOn:  dic withEvalPane: true withLabel: ('a RemoteObject: ', (self oprName)) asSymbol! !

!RemoteObject methodsFor: 'special'!
isRemote
	^true! !

!RemoteObject methodsFor: 'special'!
oprEqueals: anOperaObjectReference 
	
	^self doesNotUnderstand: (Message selector: #= argument: anOperaObjectReference )! !

!RemoteObject methodsFor: 'special'!
oprPrintString
	| aStream |
	"useful for debug"
	aStream := (String new: 64) writeStream.
	aStream nextPutAll:  'a RemoteObject( ';
		nextPutAll: self oprName, ' ) ';
		nextPutAll: 'host: ', self oprHost;
		nextPutAll: 'port: ', self oprPort printString.
	^aStream contents
	! !

!RemoteObject methodsFor: 'special'!
printOn: aStream
	"useful for debug"
	aStream nextPutAll:  'a RemoteObject( ';
		nextPutAll: self oprName, ' ) ';
		nextPutAll: 'host: ', self oprHost;
		nextPutAll: 'port: ', self oprPort printString.
	! !

!RemoteObject methodsFor: 'special'!
printString
	| stream |
	stream := WriteStream on: String new.
	self printOn: stream.
	^ stream contents! !

!RemoteObject methodsFor: 'special'!
yourself
	"for optimization"

	^self! !


!RemoteObject methodsFor: 'system callback'!
doesNotUnderstand: aMessage

	| remoteMsg |
	remoteMsg := RemoteMessageSend receiverName: self oprName selector: aMessage selector arguments: aMessage arguments.
	
	^OperaConnector default send: remoteMsg to: self oprHost port: self oprPort! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RemoteObject class
	instanceVariableNames: ' '!

!RemoteObject class methodsFor: 'instance creation'!
name: aSymbol host: aHost 
	^self name: aSymbol host: aHost port: nil! !

!RemoteObject class methodsFor: 'instance creation'!
name: aSymbol host: aHost port: aPort 
	| inst |
	inst := self new.
	inst oprName: aSymbol.
	inst oprHost: aHost.
	inst oprPort: aPort.
	^inst! !

!RemoteObject class methodsFor: 'instance creation'!
named: aSymbol
	^self name: aSymbol host: nil! !


RemoteObject subclass: #DebugRemoteObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!DebugRemoteObject methodsFor: 'system callback'!
doesNotUnderstand: aMessage

	| remoteMsg |
	remoteMsg := RemoteMessageSend receiverName: self oprName selector: aMessage selector arguments: aMessage arguments.
	
	Transcript cr; show: '++> ', remoteMsg receiverName.
	Transcript space; show: remoteMsg selector.
	Transcript space; show: remoteMsg arguments.
	
	^OperaConnector default send: remoteMsg to: self oprHost port: self oprPort! !


Object subclass: #RemoteReturnValue
	instanceVariableNames: 'value '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

!RemoteReturnValue methodsFor: 'storing'!
storeString
	^OperaObjectStream stringFromObject: self value! !


!RemoteReturnValue methodsFor: 'accessing'!
value
	^value! !

!RemoteReturnValue methodsFor: 'accessing'!
value: aValue
	value := aValue! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RemoteReturnValue class
	instanceVariableNames: ''!

!RemoteReturnValue class methodsFor: 'instance creation'!
fromString: aString
	  ^self value: (OperaObjectStream objectFromString: aString)! !

!RemoteReturnValue class methodsFor: 'instance creation'!
value: aValue
	^self new value: aValue! !


Object subclass: #SoapOperaDoc
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOAP-Opera'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SoapOperaDoc class
	instanceVariableNames: ''!

!SoapOperaDoc class methodsFor: 'examples'!
clientExample1
	| john |
	"SoapOperaDoc clientExample1"
	"try changing Cs2Person >> doSomething: to see various results"
	
	"##Client side##"
	" 0: open transcript(to see the results)"
	"OperaORB debugMode: #http."
	"OperaORB debugMode: #object."
	"OperaORB debugMode: #off."
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := RemoteObject named: #John.
	"john := RemoteObject name: #John host:<your server name>. "
	
	" 3: remote message send"
	john doSomething: 'Smalltalk!!!!'.
	
	! !

!SoapOperaDoc class methodsFor: 'examples'!
clientExample2
	| john ret |
	"SoapOperaDoc clientExample2"
	
	"##Client side##"
	" 0: open transcript(to see the results)"
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := RemoteObject named: #John.
	"john := RemoteObject name: #John host:<your server name>. "
	
	" 3: remote message send"
	ret := john doSomethingAndReturnSimple: 'Smalltalk!!!!'.
	
	" 4: remote message send (causes error)"
	"john add: #fooBarBaz"
	
	Transcript cr; show: ret printString. 
	
	! !

!SoapOperaDoc class methodsFor: 'examples'!
clientExample3
	| john  |
	"SoapOperaDoc clientExample3"
		
	"##Client side##"
	" 0: open transcript(to see the results)"
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := RemoteObject named: #John.
	"john := RemoteObject name: #John host:<your server name>. "
	
	" 3: remote message send (causing server side error)"
	john doBadly: 'not Java'
	! !

!SoapOperaDoc class methodsFor: 'examples'!
clientExample4
	| john  ret1 ret2 |
	"SoapOperaDoc clientExample4"
	
	"##Client side##"
	" 0: open transcript(to see the results)"
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := RemoteObject named: #John.
	"john := RemoteObject name: #John host:<your server name>. "
	
	" 3: remote message send"
	ret1 := john doSomethingAndReturnSelf: 'Smalltalk - but sometimes I think we may need void'.
	ret2 := john doSomethingAndReturnSelfCopy: 'Smalltalk'.

	ret1 doSomething: 'Local'. "local copy send"
	ret2 doSomething: 'Remote'. "remote"
	
	" 4: please see the difference"
	ret1 inspect.
	ret2 inspect.
	! !

!SoapOperaDoc class methodsFor: 'examples'!
clientExample5
	| john  ret1 ret2 |
	"SoapOperaDoc clientExample5"
	
	"##Client side##"
	" 0: open transcript(to see the results)"
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := RemoteObject named: #John.
	"john := RemoteObject name: #John host:<your server name>. "
	
	" 3: remote message send"
	ret1 := john doSomethingAndReturnObjRef: 'Eiffel'.
	ret2 := john doSomethingAndReturnObjValue: 'Smalltalk'.
	
	ret1 doSomething: 'Remote'. "remote"
	ret2 doSomething: 'Local'. "local copy send"
	
	" 4: please see the difference"
	ret1 inspect.
	ret2 inspect.
	! !

!SoapOperaDoc class methodsFor: 'examples'!
clientExample6
	| john paul |
	"SoapOperaDoc clientExample6"
	
	"##Client side##"
	" 0: open transcript(to see the results)"
	
	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: create remote object"
	john := RemoteObject named: #John.
	"john := RemoteObject name: #John host:<your server name>. "
	
	" 3: create object reference"
	paul := OperaObjectReference on: (Cs2Person named: 'Paul').
	
	" 4: remote message send"
	john doSomethingAndArgDoSomething: paul.
	
	! !

!SoapOperaDoc class methodsFor: 'examples'!
serverExample1
	"SoapOperaDoc serverExample1"
	"for GUI"
	"0.2b does not support this example (temporarily.)"
	
	"##Server side##"
	" 0: open transcript(to see the results)"

	" 1: start OperaORB"
	OperaORB launch.
	
	" 2: init object adapter"
	OperaORB rootOOA initialize.
	
	" 3: register object implementation"
	OperaORB rootOOA at: #John put: Cs2Person new.
	! !

!SoapOperaDoc class methodsFor: 'examples'!
serverExample2
	"SoapOperaDoc serverExample2"
	
	"##Server side##"
	" 0: open transcript(to see the results)"

	" 1: start OperaORB (optional - check your StartUpList)"
	OperaORB run.
	
	" 2: init object adapter"
	OperaORB rootOOA initialize.
	
	" 3: register object implementation"
	OperaORB rootOOA at: #John put: Cs2Person new.
	! !


!SoapOperaDoc class methodsFor: 'documentation'!
aboutSoapOpera
"## SoapOpera ## 
SoapOpera is a tiny ORB based on Soap-Http Protocol.

-Prerequisites:

 - VW XML Parser ported by Bijan Parsia(http://minnow.cc.gatech.edu/squeak/1408)
 - A refactored SOAP implementation 
        (included in this release)
        (but original is CS2 SOAP implementation. 
         http://wiki.cs.uiuc.edu/CampSmalltalk/SOAP+Smalltalk+Reference+Implementation)
 - ComSwiki (Comanche) (http://minnow.cc.gatech.edu/swiki)

Enjoy!!

-2000/12/28 0.2
-2001/12/02 0.2b
---
[:masashi | ^umezawa] 
"! !

OperaSingleton initialize!
OperaObjectAdapter initialize!